import re

from bs4 import Tag

from .base_provider import BaseProvider
from ..parser import Parser


class _ExampleProvider(Parser):

    def val_sent(self):
        sent = self.select("em", one=False)[0]
        if not sent:
            sent = self.bs.contents
        return re.search("‘?(?P<c>.+)’?", sent).group("c") if sent else None

    def val_trans(self):
        try:
            return self.select("em", one=False)[1]
        except IndexError:
            return None


class _ExpEntryProvider(Parser):

    @property
    def val_indicators(self):
        return self.select("span.indicators")

    @property
    def val_trans(self):
        trans = self.select("div.tr")
        return re.search("‘?(?P<c>.+)’?", trans).group("c") if trans else None

    @property
    def val_exp(self):
        exp = self.select("div.exg.em")
        if not exp:
            exp = self.select(".ind")
        return re.search("‘?(?P<c>.+)’?", exp).group("c") if exp else None

    @property
    def val_examples(self):
        _ = self.provider_to_list(_ExampleProvider, 'div.examples > div.exg > ul > li.ex')
        _.insert(0, _ExampleProvider(self.select(".ex", one=True, text=False)).to_dict())
        return _


class _GrambProvider(Parser):

    @property
    def val_pos(self):
        return self.select('span.pos')

    @property
    def val_exps(self):
        return self.provider_to_list(_ExpEntryProvider, 'ul.semb > li')


class _PrimaryHeaderProvider(Parser):

    def val_head_word(self):
        sup = self.select("span.hw > sup")
        return self.select("span.hw").replace(sup if sup else '', "")

    def val_trans(self):
        return self.select('span.head-translation')


class _PhraseProvider(Parser):  # class_=senseInnerWrapper

    def val_phrases(self):
        return self.select("strong.phrase", one=False)

    def val_exps(self):
        return self.provider_to_list(_ExpEntryProvider, 'ul.semb > li')


class Lexico(BaseProvider):

    @property
    def url(self):
        _ = 'definition'
        if self.seg in ('es-en',): _ = 'traducir'
        if self.seg in ('en-es',): _ = 'translate'
        if self.seg in ('es',): _ = 'definicion'
        return f"https://www.lexico.com/{self.seg}/{_}/{self.word}"

    def __init__(self, word: str, seg='es-en'):
        super(Lexico, self).__init__(word, seg)
        self.seg = seg

    @property
    def head_word(self):
        return _PrimaryHeaderProvider(self.select(".primary_homograph", text=False)).to_dict()

    @property
    def val_audio(self):
        return self.select("a.headwordAudio.rsbtn_play", text=False).audio['src']

    @property
    def val_defs(self):
        return self.provider_to_list(_GrambProvider, 'section.gramb')

    @property
    def val_phrases(self):
        phrase_title_tag: Tag = self.bs.find("h3", class_='phrases-title')
        if not phrase_title_tag:
            return []
        return self.provider_to_list(_PhraseProvider, "section.etymology.etym > div.senseInnerWrapper")



import json
import unittest

from web_dict.core.prviders.collinsdictionary import CollinsWeb


class CollinsTest(unittest.TestCase):

    def test_spanish_1(self):
        c = CollinsWeb('hacer')
        #self._p(c)

    def test_spanish_2(self):
        c = CollinsWeb('cómo')
        self._p(c)

    def test_english_1(self):
        c = CollinsWeb('INTRANSITIVE', 'english')
        #self._p(c)

    def _p(self, c):
        print(json.dumps(c.to_dict(), indent=4, ensure_ascii=False))


if __name__ == '__main__':
    unittest.main()



class NoProviderError:
    def __init__(self, dict_type: str):
        super(NoProviderError, self).__init__(f"No provider named: {dict_type}")


class NoTranslationSegmentError(Exception):
    def __init__(self, bp: str, in_: str, tar: str):
        super(NoTranslationSegmentError, self).__init__(f"No segment of {bp} for: {in_} -> {tar}")



from .base_provider import BaseProvider
from ..parser import Parser


class _PrimaryDefinitionProvider(Parser):

    def val_pos(self):
        return self.select('td.posList > a', one=False)

    def val_exp(self):
        return self.select("td > div.def").replace(f"{self.val_pos()} ", "")


class _FullDefinitionItemProvider(Parser):

    def val_pos(self):
        return self.select("h3.definition > a")

    def val_exp(self):
        return self.select("h3.definition").lstrip(self.val_pos()).strip()

    def val_examples(self):
        sentences = self.select("div.example", one=False)
        sentence_highlight = self.select("div.example > strong", one=False)
        return [
            {
                "sentence": s,
                "highlight": w
            } for s, w in zip(sentences, sentence_highlight)
        ]


class _FullDefinitionGroupProvider(Parser):

    def val_items(self):
        return self.provider_to_list(_FullDefinitionItemProvider, 'div.ordinal')


class _DefinitionsProvider(Parser):
    def val_primary(self):
        return self.provider_to_list(_PrimaryDefinitionProvider, ('tr', {}))

    def val_full(self):
        return self.provider_to_list(
            _FullDefinitionItemProvider, 'div.group > div.ordinal'
        )


class Vocabulary(BaseProvider):

    @property
    def url(self):
        return f"https://www.vocabulary.com/dictionary/{self.word}"

    def __init__(self, word: str, seg=''):
        super(Vocabulary, self).__init__(word, seg)
        self.seg = seg

    @property
    def head_word(self):
        return self.select("h1.dynamictext")

    @property
    def val_audio(self):
        audio_id = self.select("h1.dynamictext", text=False).a['data-audio']
        if audio_id:
            return f'https://audio.vocab.com/1.0/us/{audio_id}.mp3'

    @property
    def val_short_def(self):
        return self.select("p.short")

    @property
    def val_long_def(self):
        return self.select("p.long")

    # @property
    # def val_sentences(self):
    #     try:
    #         return [
    #             sd['sentence'] for sd in
    #             requests.get(f'https://corpus.vocabulary.com/api/1.0/examples.json'
    #                          f'?query={self.head_word}&maxResults=24').json()
    #             ['result']['sentences']
    #         ]
    #     except KeyError:
    #         return None

    @property
    def val_defs(self):
        return _DefinitionsProvider(
            self.bs.find('div', class_='definitions')
        ).to_dict()



import json
import unittest

from web_dict.core.factory import VocabularyDictionary


class VocabularyTest(unittest.TestCase):

    def setUp(self) -> None:
        self.c = VocabularyDictionary(word='python')

    def test_basic(self):
        self.assertIsNotNone(self.c.en(), )
        self._p(self.c.en())

    def _p(self, c):
        print(json.dumps(c, indent=4, ensure_ascii=False))


if __name__ == '__main__':
    unittest.main()



from .core.factory import CollinsDictionary, OxfordDictionary, VocabularyDictionary

LexicoDictionary = OxfordDictionary
__all__ = (CollinsDictionary, LexicoDictionary,
           OxfordDictionary, VocabularyDictionary)



import abc

from ..parser import WebParser


class BaseProvider(WebParser):
    to_dict_fields = ('head_word',)

    def __init__(self, word: str, seg: str):
        super(BaseProvider, self).__init__(word, )
        self.seg = seg

    @property
    @abc.abstractmethod
    def head_word(self):
        ...
